package de.juplo.accelerator.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;

/**
 * Über diese Annotation kann der Inhalt des <code>Last-Modified</code>-Headers
 * gesteuert werden.
 * Mit dieser Annotation können Klassen oder Methoden merkiert werden.
 * <p>
 * Wenn eine Methode annotiert wird, muss diese eine Instanz von
 * {@link HttpServletRequest} als (einziges!) Argument akzeptieren und einen
 * Wert liefern, der sich nach <code>long</code> casten lässt.
 * Die Signatur der Methode entspricht der Methode
 * {@link HttpServlet#getLastModified(javax.servlet.http.HttpServletRequest)}
 * aus dem <code>HttpServlet</code>-Interface.
 * Um das Cache-Verhalten ein existierendes Servlet, das diese Methode bereits
 * implementiert, mit Juplo-CacheControll zu verbessern, kann als erste
 * Maßnahme daher einfach diese Methode mit dieser Annotation markiert werden.
 * <p>
 * Wenn eine Klasse Annotiert wird, muss der Annotation der Wert für den
 * <code>Last-Modified</code>-Header übergeben werden.
 * Da dieser Wert somit statisch ist, macht es nur Sinn, Klassen mit dieser
 * Annotation zu markieren, die ausschließlich statische Ressourcen ausliefern,
 * die sich nur mit der Neuinstallation der Webanwendung ändern.
 * </p>
 * Über diese Annotation wird der Zeitpunkt gesteuert, zu dem die gelieferte
 * Ressource zuletzt verändert wurde.
 * Erwartet wird eine Zeitangabe in Millisekunden seit dem Unix-0-Zeitpunkt,
 * die dann an {@link HttpServletResponse#setDateHeader(String, long)}
 * weitergegeben wird.
 * <p>
 * <strong>Zu beachten:</strong>
 * Wenn zugleich die Annotation {@link CacheSeconds} verwendet wird, wird
 * die mit dieser Annotation markierte Methode nur aufgerufen, wenn die mit
 * der Annotation {@link CacheSeconds} markierte Methode einen Wert größer
 * oder gleich <code>0</code> liefert, bzw. für die mit Annotation
 * {@link CacheSeconds} markierte Klasse eine Cache-Zeit größer oder gleich
 * <code>0</code> festgelegt wurde.
 *
 * @author kai
 * @see Cacheable
 * @see CacheSeconds
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.TYPE, ElementType.METHOD })
public @interface LastModified {
  long value() default 0;
}
